@@ -39,3 +39,29 @@ TEST_CASE("DELogisticRegressionTest", "[DETest]")
       coordinates);
   REQUIRE(testAcc == Approx(100.0).epsilon(0.006)); // 0.6% error tolerance.
 }
+
+/**
+ * Train and test a logistic regression function using DE optimizer.  Use
+ * arma::fmat.
+ */
+TEST_CASE("DELogisticRegressionFMatTest", "[DETest]")
+{
+  arma::fmat data, testData, shuffledData;
+  arma::Row<size_t> responses, testResponses, shuffledResponses;
+
+  LogisticRegressionTestData(data, testData, shuffledData, responses,
+      testResponses, shuffledResponses);
+  LogisticRegression<arma::fmat> lr(shuffledData, shuffledResponses, 0.5);
+
+  DE opt(200, 1000, 0.6, 0.8, 1e-5);
+  arma::fmat coordinates = lr.GetInitialPoint();
+  opt.Optimize(lr, coordinates);
+
+  // Ensure that the error is close to zero.
+  const float acc = lr.ComputeAccuracy(data, responses, coordinates);
+  REQUIRE(acc == Approx(100.0).epsilon(0.03)); // 3% error tolerance.
+
+  const float testAcc = lr.ComputeAccuracy(testData, testResponses,
+      coordinates);
+  REQUIRE(testAcc == Approx(100.0).epsilon(0.06)); // 6% error tolerance.
+}
