@@ -22,17 +22,39 @@ using namespace ens::test;
  */
 TEST_CASE("FuncFWTest", "[LineSearchTest]")
 {
-  vec x1 = zeros<vec>(3);
-  vec x2;
+  mat x1 = zeros<mat>(3, 1);
+  mat x2;
   x2 << 0.2 << 0.4 << 0.6;
+  x2 = x2.t();
 
-  TestFuncFW f;
+  TestFuncFW<> f;
   LineSearch s;
 
   double result = s.Optimize(f, x1, x2);
 
   REQUIRE(result == Approx(0.0).margin(1e-10));
-  REQUIRE((x2[0] - 0.1) == Approx(0.0).margin(1e-10));
-  REQUIRE((x2[1] - 0.2) == Approx(0.0).margin(1e-10));
-  REQUIRE((x2[2] - 0.3) == Approx(0.0).margin(1e-10));
+  REQUIRE((x2(0) - 0.1) == Approx(0.0).margin(1e-10));
+  REQUIRE((x2(1) - 0.2) == Approx(0.0).margin(1e-10));
+  REQUIRE((x2(2) - 0.3) == Approx(0.0).margin(1e-10));
+}
+
+/**
+ * Simple test of Line Search with TestFuncFW function.  Use arma::fmat.
+ */
+TEST_CASE("FuncFWFMatTest", "[LineSearchTest]")
+{
+  fmat x1 = zeros<fmat>(3, 1);
+  fmat x2;
+  x2 << 0.2 << 0.4 << 0.6;
+  x2 = x2.t();
+
+  TestFuncFW<arma::fmat> f;
+  LineSearch s;
+
+  float result = s.Optimize(f, x1, x2);
+
+  REQUIRE(result == Approx(0.0).margin(1e-5));
+  REQUIRE((x2(0) - 0.1) == Approx(0.0).margin(1e-5));
+  REQUIRE((x2(1) - 0.2) == Approx(0.0).margin(1e-5));
+  REQUIRE((x2(2) - 0.3) == Approx(0.0).margin(1e-5));
 }
